'==========================================================================
'
'
' NAME: CSV Generator
'
' AUTHOR: Ben Cook, Dell Software
' DATE  : 13/05/13
'
' COMMENT:  Tool to scan a folder structure searching for packages to import
'			and generate a CSV file for use with ChangeBASE 5.x
'
' Usage:	Run the script and answer all the prompts.
'
'==========================================================================
Option Explicit

' Output filename
Dim strCSVFile : strCSVFile = "ChangeBASE CSV.csv"
Dim strAppID : strAppID = "None" ' AppID for MSIs with no AppID

' Debug
Dim intDebug : intDebug = 0

' Global variabless
Dim objFSO : Set objFSO = CreateObject("Scripting.FileSystemObject")
Dim objWSHShell : Set objWSHShell = CreateObject("WScript.Shell")
Dim objErrorLog ' Log file
Dim objCSVFile ' CSV
Dim intCSVRowCount : intCSVRowCount = 0 ' Count the rows added
Dim bolMultipleMSTs ' Add multiple MSTs
Dim intSkippedMSIs : intSkippedMSIs = 0 ' Count for skipped MSIs
Dim strErrorLog
Dim strText ' Used for messages
Dim strSourceFolder ' Source path

' Get script path
Dim strScriptPath : strScriptPath = Left(WScript.ScriptFullName, InStrRev(WScript.ScriptFullName, "\"))

'Set Temp folder
Dim strTempFolder : strTempFolder = objWSHShell.ExpandEnvironmentStrings("%Temp%" & "\")

'Set output folder
Dim strOutputFolder : strOutputFolder = objWSHShell.ExpandEnvironmentStrings("%UserProfile%" & "\Desktop\")

'Create log file, add start time/date and a blank line
CreateLogFile

' Start
WriteToLog "", "Start"

' Call the main function
Main

' Finish
WriteToLog "", "End"

'----------------------------------------------------------------------------------------------------------------------
Function Main

	Dim strRegistryKey : strRegistryKey = "HKCU\Software\CSV Generator"
	Dim strSourceRegistry : strSourceRegistry = strRegistryKey & "\Source Folder"
	Dim intLevelsRegistry : intLevelsRegistry = strRegistryKey & "\Folder Levels"
	Dim intFolderAppIDRegistry : intFolderAppIDRegistry = strRegistryKey & "\Folder AppID"
' 	Dim strFileTypesRegistry : strFileTypesRegistry = strRegistryKey & "\File Types"
'	Dim bolAutoExcludeExesRegistry : bolAutoExcludeExesRegistry = strRegistryKey & "\Auto Exclude Exes"
	
	' Check if folder locations and levels have been written to registry
	On Error Resume Next
	strSourceFolder = objWSHShell.RegRead(strSourceRegistry) ' Global variable
	Dim intLevels : intLevels = objWSHShell.RegRead(intLevelsRegistry)
	Dim intFolderAppID : intFolderAppID = objWSHShell.RegRead(intFolderAppIDRegistry)	
' 	Dim strFileTypes : strFileTypes = objWSHShell.RegRead(strFileTypesRegistry)
'	Dim bolAutoExcludeExes : bolAutoExcludeExes = objWSHShell.RegRead(bolAutoExcludeExesRegistry)
	On Error Goto 0
	
	' Source folder
	strSourceFolder = InputBox("Please enter the source path", "Source Path", strSourceFolder)
	' Make sure we have a trailing backslash:
	If Right(strSourceFolder, 1) = "\" Then strSourceFolder = Left(strSourceFolder, Len(strSourceFolder) - 1)
	If Len(strSourceFolder) < 2 Then 
		WriteToLog "No source folder was entered.", "Error"
		WScript.Quit
	End If
	
	' Check folder exists
	If Not objFSO.FolderExists(strSourceFolder) Then 
		strText = "Source folder could not be found. This script will now exit."
		Msgbox strText, vbExclamation, "Error!"
		WriteToLog strText, "Error"
		Wscript.Quit
	End If
	
	' Add MSIs with multiple MSTs?
	If MsgBox("Do you want to add MSIs with multiple MSTs in the same folder?", VBYesNo + vbQuestion, "Multiple MSTs?") = VBYes Then
		bolMultipleMSTs = True
	Else
		bolMultipleMSTs = False
	End If
	
	' How many folder levels to scan?
	intLevels = InputBox("How many folders deep do you want to scan (leave blank for unlimited)?" & VbCr & VbCr & "Examples:" & VbCr & VbCr & _
		"1 folder deep:" & VbCr & strSourceFolder & "\Folder 1\" & VbCr & VbCr & _
		"2 folders deep:" & VbCr & strSourceFolder & "\Folder 1\Folder 2\" & VbCr & VbCr & _
		"3 folders deep:" & VbCr & strSourceFolder & "\Folder 1\Folder 2\Folder3\", "Folder Levels", intLevels)
	If intLevels = "" Then intLevels = 260 ' Max no of subfolders
	If Not IsNumeric(intLevels) Then 
		WriteToLog "Invalid folder level specified: " & intLevels, "Error"
		WScript.Quit
	End If
	
	' Which folder should be used for the AppID?
	intFolderAppID = InputBox("Which folder should be used for the AppID (leave blank for none)?" & VbCr & VbCr & "Examples:" & VbCr & VbCr & _
		"Enter 1 for:" & VbCr & strSourceFolder & "\Folder 1\" & VbCr & VbCr & _
		"Enter 2 for:" & VbCr & strSourceFolder & "\Folder 1\Folder 2\" & VbCr & VbCr & _
		"Enter 3 for:" & VbCr & strSourceFolder & "\Folder 1\Folder 2\Folder3\" & VbCr & "etc", "AppID Folder", intFolderAppID)
	If Not IsNumeric(intFolderAppID) Then 
		WriteToLog "Invalid AppID folder specified: " & intFolderAppID, "Error"
		WScript.Quit
	End If
	
' 	' What file types to look for?
' 	strFileTypes = InputBox("Please enter the file types to search For (separate with semi-colon)" & VbCr & VbCr & _
' 	"Eg. msi;exe", "File Types", strFileTypes)
' 	If strFileTypes = "" Then 
' 		WriteToLog "No file types were entered.", "Error"
' 		WScript.Quit
' 	End If
	
	WriteToLog "Source folder: " & strSourceFolder, ""
	WriteToLog "Folders levels to scan: " & intLevels, ""
	WriteToLog "Folder for AppID: " & intFolderAppID, ""
' 	WriteToLog "File types to search for: " & strFileTypes, ""
	
	' Check folder exists
	If Not objFSO.FolderExists(strSourceFolder) Then 
		strText = "Source folder could not be found. This script will now exit."
		WriteToLog strText, "Error"
		Wscript.Quit
	End If
	
	' Write path to registry
	objWSHShell.RegWrite strSourceRegistry, strSourceFolder
	If intLevels = "260" Then 
		objWSHShell.RegWrite intLevelsRegistry, ""
	Else
		objWSHShell.RegWrite intLevelsRegistry, intLevels
	End If
	objWSHShell.RegWrite intFolderAppIDRegistry, intFolderAppID
	
	' Check if CSV already exists
	If objFSO.FileExists(strOutputFolder & strCSVFile) Then
		If MsgBox("This CSV file already exists and will be overwritten: " & strOutputFolder & strCSVFile & _
			VbCr & VbCr & "OK to continue?", VBYesNo + vbExclamation, "CSV File Already Exists") = VBNo Then 
			WriteToLog "User chose not to overwrite CSV file. Script will exit.", ""
			Wscript.Quit	
		End If
	End If
	
	' Create CSV file
	Call CreateCSVFile
	
	' Call main function
	Call FolderScan(strSourceFolder, intLevels, intFolderAppID)
	
End Function
'----------------------------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------------------------
Function FolderScan(strFolder, intLevels, intFolderAppID)
	'Msgbox "FolderScan: " & strFolder
	
	' Get folder depth we're currently in
	Dim intCurrentFolderDepth : intCurrentFolderDepth = Len(strFolder) - Len(Replace(strFolder, "\", "")) - 1
	Dim intSourceFolderDepth : intSourceFolderDepth = Len(strSourceFolder) - Len(Replace(strSourceFolder, "\", "")) - 1
	Dim intRelativeDepth : intRelativeDepth = intCurrentFolderDepth - intSourceFolderDepth
	'If Left(strFolder, 2) = "\\" Then intFolderDepth = intFolderDepth - 3 ' To handle UNCs
	
	' Check if we're at the AppID folder, set strAppID (global variable)
	If CInt(intRelativeDepth) = CInt(intFolderAppID) Then 
		strAppID = Right(strFolder, Len(strFolder) - InStrRev(strFolder, "\"))
		strAppID = Replace(strAppID, ".", "_")
	Elseif CInt(intRelativeDepth) < CInt(intFolderAppID) Then ' Required if we are now shallower
		strAppID = "None"
	End If

	'MsgBox strFolder & VbCr & "Source depth: " & intSourceFolderDepth & VbCr & "Current depth: " & _
	'	intCurrentFolderDepth & VbCr & "AppID folder: " & intFolderAppID & VbCr & VbCr & "AppID: " & strAppID
	
	' Local variable
	Dim intFolderDepthLocal : intFolderDepthLocal = intLevels
	
	Dim objFolder : Set objFolder = objFSO.GetFolder(strFolder)
	Dim colFiles : Set colFiles = objFolder.Files
	
	Dim objFile
	' Check if MSI is present
	On Error Resume Next ' The next line may raise an error
	Dim arrMSI() ' Array for holding MSI paths (there may be multiple MSIs in a folder)
	Dim strMST ' Path to MST (only one MST used per folder)
	Dim intMST ' Counter
	For Each objFile In colFiles
		If Err.Number <> 0 Then MsgBox "Unable to access the file: " & objFile.Path,, "Error"
		Err.Clear
		On Error Goto 0
		' Look for MSIs
		If LCase(Right(objFile.Path, 4)) = ".msi" Then 
			'MsgBox "File: " & objFile.Path
			If CheckArrayElements(arrMSI) = -1 Then
				' Array is empty, redim for first element
				ReDim Preserve arrMSI(0)
			Else
				' Array already has elements
				ReDim Preserve arrMSI(UBound(arrMSI) + 1)
			End If
			' Add MSI to array
			arrMSI(UBound(arrMSI)) = objFile.Path
		' Look for MSTs
		Elseif LCase(Right(objFile.Path, 4)) = ".mst" Then 
			strMST = strMST & Chr(34) & objFile.Path & Chr(34) & ";"
			intMST = intMST + 1
		End If
	Next
	
	'Trim ; off MST string
	If Len(strMST) > 0 Then strMST = Left(strMST, Len(strMST) - 1)
	
	If CheckArrayElements(arrMSI) >= 0 And intMST = 0 Then
		'MsgBox "MSIs and no MSTs detected in folder: " & strFolder
		Dim x
		For x = 0 To UBound(arrMSI)
			Call AddMSItoCSV(arrMSI(x), "", strAppID)
		Next
	ElseIf CheckArrayElements(arrMSI) = 0 And intMST = 1 Then 
		'MsgBox "One MSI and one MST detected in folder: " & strFolder
		Call AddMSItoCSV(arrMSI(0), "TRANSFORMS=" & strMST, strAppID)
	ElseIf bolMultipleMSTs and intMST > 1 Then
		' Multiple MSTs - add
		Dim y
		For y = 0 To UBound(arrMSI)	
			Call AddMSItoCSV(arrMSI(y), "TRANSFORMS=" & strMST, strAppID)
		Next
	ElseIf Not bolMultipleMSTs and intMST > 1 Then
		' Multiple MSTs - skip
		Dim z
		For z = 0 To UBound(arrMSI)
			WriteToLog "MSI skipped: " & arrMSI(z) & ". Transforms in folder: " & intMST, ""
			intSkippedMSIs = intSkippedMSIs + 1
		Next
	End If
	
	'Loop through subfolders
	If intFolderDepthLocal > 0 Then
		intFolderDepthLocal = intFolderDepthLocal - 1
		Dim objSubfolder
		For Each objSubfolder in objFolder.SubFolders
	       	Call FolderScan(objSubfolder.Path, intFolderDepthLocal, intFolderAppID)
	    Next
	End If
	
	Set objFolder = Nothing
	Set colFiles = Nothing

End Function
'----------------------------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------------------------
Function CreateLogFile

	strErrorLog = strTempFolder & "CSV Generator " & Year(Now) & "." & Month(Now) & "." & Day(Now) & " " & Hour(Now) & _
	"." & Minute(Now) & "." & Second(Now) & ".log"

	Set objErrorLog = objFSO.CreateTextFile(strErrorLog, True)

End Function
'----------------------------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------------------------
Function WriteToLog(strText, strMode)

	If intDebug = 1 Then Msgbox "Function WriteToLog is running..." & VbCr & "strText: " & strText & VbCr & "strMode: " & strMode
	
	If LCase(strMode) = "start" Then
		objErrorLog.WriteLine "Script started on " & Date & " at " & Time
		objErrorLog.WriteBlankLines 1
	ElseIf LCase(strMode) = "end" Then
		objErrorLog.WriteLine intCSVRowCount & " items added to CSV file."
		objErrorLog.WriteLine intSkippedMSIs & " MSIs were skipped. See the log file in the %TEMP% folder for full details."
		objErrorLog.WriteLine "Script ended on " & Date & " at " & Time
		If MsgBox(intCSVRowCount & " items added to CSV file. " & VbCr & VbCr & intSkippedMSIs & " MSI(s) not added. " & _
			"See the log file in the %TEMP% folder for full details." & VbCr & VbCr & _
			"Would you like to view the CSV file: " & strOutputFolder & strCSVFile & "?", VBYesNo, "My work here is done") = VBYes Then _
			objWSHShell.Run("Notepad.exe " & strOutputFolder & strCSVFile)
	ElseIf LCase(strMode) = "error" Then
		objErrorLog.WriteLine "Error: " & strText
		WriteToLog "", "Blank"
		MsgBox "An error occurred: " & strText
		WriteToLog "", "End"
	ElseIf LCase(strMode) = "blank" Then
		objErrorLog.WriteBlankLines 1
	Else
		objErrorLog.WriteLine(strText)
	End If

End Function
'----------------------------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------------------------
Function CreateCSVFile

	On Error Resume Next
	Set objCSVFile = objFSO.CreateTextFile(strOutputFolder & strCSVFile, True)
	If Err.Number = 0 Then
		WriteToLog "CSV created successfully: " & strOutputFolder & strCSVFile, ""
	Else
		WriteToLog "Unable to create the CSV file: " & strOutputFolder & strCSVFile & ". Error: " & Err.Description, "Error"
	End If
	On Error Goto 0
	objCSVFile.WriteLine "Executable File Name,Command Line Arguments,Support Files (Y/N),AutoComplete (Y/N)," & _
		"ProductName,ProductVersion" & ",MSI OutPut Folder,Launch Application,Manufacturer,View,AppID"
	
End Function
'----------------------------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------------------------
Function AddMSItoCSV(strMSI, strMST, strAppID)

	objCSVFile.WriteLine strMSI & "," & strMST & ",,,,,,,,," & strAppID
	intCSVRowCount = intCSVRowCount + 1

End Function
'----------------------------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------------------------
Function CheckArrayElements(strArray)

	On Error Resume Next
	dim intTest : intTest = UBound(strArray)
	If Err.Number = 0 Then 
		CheckArrayElements = intTest
	Else
		CheckArrayElements = -1
	End If
	Err.Clear
	On Error Goto 0

End Function
'----------------------------------------------------------------------------------------------------------------------